@@ -18,7 +18,11 @@ class ExtractMsgFormatter(logging.Formatter):
         return record.msg
 
 
-def request_service_account_key(*, project_credentials: dict = utils.get_project_credentials()):
+def request_service_account_key(
+    *,
+    project_credentials: dict = utils.get_project_credentials(),
+    requests: requests = requests,
+):
     '''
     Request the latest GCP service account key and write it to a temp file.
     '''
@@ -54,29 +58,44 @@ def get_stderr_logger() -> logging.Logger:
     return logger
 
 
-def get_gcp_logger() -> logging.Logger:
+def get_gcp_logging_client() -> google.cloud.logging.Client:
     '''
-    Initialize a GCP logger.
-
-    TODO: extra the client initialization logic
+    Initialize a service account key and a Google Logging client
     '''
     key_file_path = request_service_account_key()
     if not key_file_path:
         utils.report_client_error('Unable to request service credentials. Data will not be be sent to DCYD.')
-        return get_stderr_logger()
+        return None
 
-    client = google.cloud.logging.Client.from_service_account_json(key_file_path)
+    return google.cloud.logging.Client.from_service_account_json(key_file_path)
 
+def get_gcp_logging_handler(
+    client: google.cloud.logging.Client,
+) -> google.cloud.logging.handlers.handlers.CloudLoggingHandler:
     handler = google.cloud.logging.handlers.handlers.CloudLoggingHandler(
         client=client,
         name='mpm-client-log',  # NOTE: This logger name is critical for downstream data processing. Do not change
         transport=gcl_patch.BackgroundThreadTransport,
     )
+    return handler
+
+
+def get_gcp_logger(client: google.cloud.logging.Client) -> logging.Logger:
+    '''
+    Initialize a GCP logger.
+
+    '''
+
+    if not client:
+        return get_stderr_logger()
+
+    handler = get_gcp_logging_handler(client)
     handler.setFormatter(ExtractMsgFormatter())
 
     logger = logging.getLogger('mpm-client-log')
-    logger.setLevel(logging.NOTSET)
+    logger.setLevel(logging.INFO)
     logger.addHandler(handler)
+
     return logger
 
 
@@ -94,6 +113,7 @@ def get_file_logger(*, log_file_path: str = os.getenv('DCYD_CLIENT_FILE_LOGGER')
     logger.addHandler(handler)
     return logger
 
+
 def get_http_logger(*, endpoint: str = os.getenv('DCYD_CLIENT_HTTP_LOGGER')) -> logging.Logger:
     '''
     Initialize an HTTP logger
@@ -106,27 +126,32 @@ def get_http_logger(*, endpoint: str = os.getenv('DCYD_CLIENT_HTTP_LOGGER')) ->
     return logger
 
 
-FILE_LOGGER = get_file_logger()
-HTTP_LOGGER = get_http_logger()
-GCP_LOGGER = get_gcp_logger()
+_GCP_CLIENT = get_gcp_logging_client() # Create/authenticate the client now, but create the handler w/in
+_LOGGERS = {
+    'file': get_file_logger(),
+    'http': get_http_logger(),
+    'gcp': None,
+}
 
 
-def log_struct(record: dict):
+def log_struct(record: dict, loggers: dict= _LOGGERS):
     '''
     Write a dictionary to GCP Logging
     '''
-    if FILE_LOGGER:
-        FILE_LOGGER.info(json.dumps({
+    if loggers.get('file'):
+        loggers.get('file').info(json.dumps({
             'insertId': utils.generate_uid(),
             'jsonPayload': record,
             'timestamp': datetime.datetime.utcnow().isoformat() + 'Z',
             'receiveTimestamp': datetime.datetime.utcnow().isoformat() + 'Z',
         }))
-    if HTTP_LOGGER:
-        HTTP_LOGGER.info({
+    if loggers.get('http'):
+        loggers.get('http').info({
             'insertId': utils.generate_uid(),
             'jsonPayload': record,
             'timestamp': datetime.datetime.utcnow().isoformat() + 'Z',
             'receiveTimestamp': datetime.datetime.utcnow().isoformat() + 'Z',
         })
-    return GCP_LOGGER.log(level=logging.INFO, msg=record)
+    if not loggers.get('gcp'):
+        loggers['gcp'] = get_gcp_logger(client=_GCP_CLIENT)
+    return loggers.get('gcp').log(level=logging.INFO, msg=record)
